@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2017 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -18,12 +18,14 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Network plugin."""
+from __future__ import unicode_literals
 
 import base64
 import operator
 
 from glances.timer import getTimeSinceLastUpdate
 from glances.plugins.glances_plugin import GlancesPlugin
+from glances.compat import n, u, b, nativestr
 
 import psutil
 
@@ -35,43 +37,34 @@ snmp_oid = {'default': {'interface_name': '1.3.6.1.2.1.2.2.1.2',
                         'cumulative_tx': '1.3.6.1.2.1.2.2.1.16'}}
 
 # Define the history items list
-# All items in this list will be historised if the --enable-history tag is set
-# 'color' define the graph color in #RGB format
 items_history_list = [{'name': 'rx',
                        'description': 'Download rate per second',
-                       'color': '#00FF00',
                        'y_unit': 'bit/s'},
                       {'name': 'tx',
                        'description': 'Upload rate per second',
-                       'color': '#FF0000',
                        'y_unit': 'bit/s'}]
 
 
 class Plugin(GlancesPlugin):
-
     """Glances network plugin.
 
     stats is a list
     """
 
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args, items_history_list=items_history_list)
+        super(Plugin, self).__init__(args=args,
+                                     config=config,
+                                     items_history_list=items_history_list,
+                                     stats_init_value=[])
 
         # We want to display the stat in the curse interface
         self.display_curse = True
 
-        # Init the stats
-        self.reset()
-
     def get_key(self):
         """Return the key of the list."""
         return 'interface_name'
 
-    def reset(self):
-        """Reset/init the stats."""
-        self.stats = []
-
     @GlancesPlugin._check_decorator
     @GlancesPlugin._log_result_decorator
     def update(self):
@@ -79,26 +72,27 @@ class Plugin(GlancesPlugin):
 
         Stats is a list of dict (one dict per interface)
         """
-        # Reset stats
-        self.reset()
+        # Init new stats
+        stats = self.get_init_value()
 
         if self.input_method == 'local':
             # Update stats using the standard system lib
 
-            # Grab network interface stat using the PsUtil net_io_counter method
+            # Grab network interface stat using the psutil net_io_counter method
             try:
                 netiocounters = psutil.net_io_counters(pernic=True)
-            except UnicodeDecodeError:
+            except UnicodeDecodeError as e:
+                logger.debug('Can not get network interface counters ({})'.format(e))
                 return self.stats
 
-            # New in PsUtil 3.0
-            # - import the interface's status (issue #765)
-            # - import the interface's speed (issue #718)
+            # Grab interface's status (issue #765)
+            # Grab interface's speed (issue #718)
             netstatus = {}
             try:
                 netstatus = psutil.net_if_stats()
-            except (AttributeError, OSError):
-                pass
+            except OSError as e:
+                # see psutil #797/glances #1106
+                logger.debug('Can not get network interface status ({})'.format(e))
 
             # Previous network interface stats are stored in the network_old variable
             if not hasattr(self, 'network_old'):
@@ -107,56 +101,51 @@ class Plugin(GlancesPlugin):
                     self.network_old = netiocounters
                 except (IOError, UnboundLocalError):
                     pass
-            else:
-                # By storing time data we enable Rx/s and Tx/s calculations in the
-                # XML/RPC API, which would otherwise be overly difficult work
-                # for users of the API
-                time_since_update = getTimeSinceLastUpdate('net')
+                return self.stats
 
-                # Loop over interfaces
-                network_new = netiocounters
-                for net in network_new:
-                    # Do not take hidden interface into account
-                    if self.is_hide(net):
-                        continue
-                    try:
-                        cumulative_rx = network_new[net].bytes_recv
-                        cumulative_tx = network_new[net].bytes_sent
-                        cumulative_cx = cumulative_rx + cumulative_tx
-                        rx = cumulative_rx - self.network_old[net].bytes_recv
-                        tx = cumulative_tx - self.network_old[net].bytes_sent
-                        cx = rx + tx
-                        netstat = {
-                            'interface_name': net,
-                            'time_since_update': time_since_update,
-                            'cumulative_rx': cumulative_rx,
-                            'rx': rx,
-                            'cumulative_tx': cumulative_tx,
-                            'tx': tx,
-                            'cumulative_cx': cumulative_cx,
-                            'cx': cx}
-                    except KeyError:
-                        continue
-                    else:
-                        # Optional stats (only compliant with PsUtil 3.0+)
-                        # Interface status
-                        try:
-                            netstat['is_up'] = netstatus[net].isup
-                        except (KeyError, AttributeError):
-                            pass
-                        # Interface speed in Mbps, convert it to bps
-                        # Can be always 0 on some OS
-                        try:
-                            netstat['speed'] = netstatus[net].speed * 1048576
-                        except (KeyError, AttributeError):
-                            pass
+            # By storing time data we enable Rx/s and Tx/s calculations in the
+            # XML/RPC API, which would otherwise be overly difficult work
+            # for users of the API
+            time_since_update = getTimeSinceLastUpdate('net')
 
-                        # Finaly, set the key
-                        netstat['key'] = self.get_key()
-                        self.stats.append(netstat)
+            # Loop over interfaces
+            network_new = netiocounters
+            for net in network_new:
+                # Do not take hidden interface into account
+                # or KeyError: 'eth0' when interface is not connected #1348
+                if self.is_hide(net) or net not in netstatus:
+                    continue
+                try:
+                    cumulative_rx = network_new[net].bytes_recv
+                    cumulative_tx = network_new[net].bytes_sent
+                    cumulative_cx = cumulative_rx + cumulative_tx
+                    rx = cumulative_rx - self.network_old[net].bytes_recv
+                    tx = cumulative_tx - self.network_old[net].bytes_sent
+                    cx = rx + tx
+                    netstat = {'interface_name': n(net),
+                               'time_since_update': time_since_update,
+                               'cumulative_rx': cumulative_rx,
+                               'rx': rx,
+                               'cumulative_tx': cumulative_tx,
+                               'tx': tx,
+                               'cumulative_cx': cumulative_cx,
+                               'cx': cx,
+                               # Interface status
+                               'is_up': netstatus[net].isup,
+                               # Interface speed in Mbps, convert it to bps
+                               # Can be always 0 on some OSes
+                               'speed': netstatus[net].speed * 1048576,
+                               # Set the key for the dict
+                               'key': self.get_key()
+                               }
+                except KeyError:
+                    continue
+                else:
+                    # Append the interface stats to the list
+                    stats.append(netstat)
 
-                # Save stats to compute next bitrate
-                self.network_old = network_new
+            # Save stats to compute next bitrate
+            self.network_old = network_new
 
         elif self.input_method == 'snmp':
             # Update stats using SNMP
@@ -218,11 +207,14 @@ class Plugin(GlancesPlugin):
                         continue
                     else:
                         netstat['key'] = self.get_key()
-                        self.stats.append(netstat)
+                        stats.append(netstat)
 
                 # Save stats to compute next bitrate
                 self.network_old = network_new
 
+        # Update the stats
+        self.stats = stats
+
         return self.stats
 
     def update_views(self):
@@ -264,15 +256,10 @@ class Plugin(GlancesPlugin):
             return ret
 
         # Max size for the interface name
-        if max_width is not None and max_width >= 23:
-            # Interface size name = max_width - space for interfaces bitrate
-            ifname_max_width = max_width - 14
-        else:
-            ifname_max_width = 9
+        name_max_width = max_width - 12
 
-        # Build the string message
         # Header
-        msg = '{:{width}}'.format('NETWORK', width=ifname_max_width)
+        msg = '{:{width}}'.format('NETWORK', width=name_max_width)
         ret.append(self.curse_add_line(msg, "TITLE"))
         if args.network_cumul:
             # Cumulative stats
@@ -298,7 +285,7 @@ class Plugin(GlancesPlugin):
                 msg = '{:>7}'.format('Tx/s')
                 ret.append(self.curse_add_line(msg))
         # Interface list (sorted by name)
-        for i in sorted(self.stats, key=operator.itemgetter(self.get_key())):
+        for i in self.sorted_stats():
             # Do not display interface in down state (issue #765)
             if ('is_up' in i) and (i['is_up'] is False):
                 continue
@@ -308,9 +295,9 @@ class Plugin(GlancesPlugin):
             ifname = self.has_alias(i['interface_name'])
             if ifname is None:
                 ifname = ifrealname
-            if len(ifname) > ifname_max_width:
+            if len(ifname) > name_max_width:
                 # Cut interface name if it is too long
-                ifname = '_' + ifname[-ifname_max_width + 1:]
+                ifname = '_' + ifname[-name_max_width + 1:]
 
             if args.byte:
                 # Bytes per second (for dummy)
@@ -334,7 +321,7 @@ class Plugin(GlancesPlugin):
 
             # New line
             ret.append(self.curse_new_line())
-            msg = '{:{width}}'.format(ifname, width=ifname_max_width)
+            msg = '{:{width}}'.format(ifname, width=name_max_width)
             ret.append(self.curse_add_line(msg))
             if args.network_sum:
                 msg = '{:>14}'.format(sx)
