@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2015 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -17,45 +17,54 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-# Import system libs
+"""Now (current date) plugin."""
+
+from time import tzname, localtime
 from datetime import datetime
 
-# Import Glances libs
 from glances.plugins.glances_plugin import GlancesPlugin
 
 
 class Plugin(GlancesPlugin):
-
     """Plugin to get the current date/time.
 
     stats is (string)
     """
 
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the plugin."""
-        GlancesPlugin.__init__(self, args=args)
+        super(Plugin, self).__init__(args=args, config=config)
 
         # We want to display the stat in the curse interface
         self.display_curse = True
 
         # Set the message position
-        self.set_align('bottom')
+        self.align = 'bottom'
+
+    def reset(self):
+        """Reset/init the stats."""
+        self.stats = ''
 
     def update(self):
         """Update current date/time."""
         # Had to convert it to string because datetime is not JSON serializable
-        self.stats = datetime.now().strftime(_("%Y-%m-%d %H:%M:%S"))
+        self.stats = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
+        # Add the time zone (issue #1249 and issue #1337)
+        if 'tmzone' in localtime():
+            self.stats += ' {}'.format(localtime().tm_zone)
+        elif len(tzname) > 0:
+            self.stats += ' {}'.format(tzname[1])
 
         return self.stats
 
-    def msg_curse(self, args=None):
+    def msg_curse(self, args=None, max_width=None):
         """Return the string to display in the curse interface."""
         # Init the return message
         ret = []
 
         # Build the string message
         # 23 is the padding for the process list
-        msg = '{0:23}'.format(self.stats)
+        msg = '{:23}'.format(self.stats)
         ret.append(self.curse_add_line(msg))
 
         return ret
