@@ -41,6 +41,7 @@ class GlancesPlugin(object):
 
     def __init__(self,
                  args=None,
+                 config=None,
                  items_history_list=None,
                  stats_init_value={}):
         """Init the plugin of plugins class.
@@ -66,7 +67,8 @@ class GlancesPlugin(object):
         :stats_init_value: Default value for a stats item
         """
         # Plugin name (= module name without glances_)
-        self.plugin_name = self.__class__.__module__[len('glances_'):]
+        pos = self.__class__.__module__.find('glances_') + len('glances') + 1
+        self.plugin_name = self.__class__.__module__[pos:]
         # logger.debug("Init plugin %s" % self.plugin_name)
 
         # Init the args
@@ -83,8 +85,11 @@ class GlancesPlugin(object):
         self.items_history_list = items_history_list
         self.stats_history = self.init_stats_history()
 
-        # Init the limits dictionnary
+        # Init the limits (configuration keys) dictionnary
+        logger.debug('Load section {} in {}'.format(self.plugin_name,
+                                                    config.config_file_paths()))
         self._limits = dict()
+        self.load_limits(config=config)
 
         # Init the actions
         self.actions = GlancesActions(args=args)
@@ -149,19 +154,19 @@ class GlancesPlugin(object):
         except UnicodeDecodeError:
             return json.dumps(d, ensure_ascii=False)
 
-    def _history_enable(self):
+    def history_enable(self):
         return self.args is not None and not self.args.disable_history and self.get_items_history_list() is not None
 
     def init_stats_history(self):
         """Init the stats history (dict of GlancesAttribute)."""
-        if self._history_enable():
+        if self.history_enable():
             init_list = [a['name'] for a in self.get_items_history_list()]
             logger.debug("Stats history activated for plugin {} (items: {})".format(self.plugin_name, init_list))
         return GlancesHistory()
 
     def reset_stats_history(self):
         """Reset the stats history (dict of GlancesAttribute)."""
-        if self._history_enable():
+        if self.history_enable():
             reset_list = [a['name'] for a in self.get_items_history_list()]
             logger.debug("Reset history for plugin {} (items: {})".format(self.plugin_name, reset_list))
             self.stats_history.reset()
@@ -174,7 +179,7 @@ class GlancesPlugin(object):
         else:
             item_name = self.get_key()
         # Build the history
-        if self.get_export() and self._history_enable():
+        if self.get_export() and self.history_enable():
             for i in self.get_items_history_list():
                 if isinstance(self.get_export(), list):
                     # Stats is a list of data
@@ -390,7 +395,8 @@ class GlancesPlugin(object):
             try:
                 # Source:
                 # http://stackoverflow.com/questions/4573875/python-get-index-of-dictionary-item-in-list
-                return self._json_dumps({item: map(itemgetter(item), self.stats)})
+                # But https://github.com/nicolargo/glances/issues/1401
+                return self._json_dumps({item: list(map(itemgetter(item), self.stats))})
             except (KeyError, ValueError) as e:
                 logger.error("Cannot get item {} ({})".format(item, e))
                 return None
@@ -494,6 +500,7 @@ class GlancesPlugin(object):
             return False
 
         # Read the global section
+        # @TODO: not optimized because this section is loaded for each plugin...
         if config.has_section('global'):
             self._limits['history_size'] = config.get_float_value('global', 'history_size', default=28800)
             logger.debug("Load configuration key: {} = {}".format('history_size', self._limits['history_size']))
@@ -725,7 +732,7 @@ class GlancesPlugin(object):
         # Return the action list
         return log_tag[0].lower() == 'true'
 
-    def get_conf_value(self, value, header="", plugin_name=None):
+    def get_conf_value(self, value, header="", plugin_name=None, default=[]):
         """Return the configuration (header_) value for the current plugin.
 
         ...or the one given by the plugin_name var.
@@ -741,7 +748,7 @@ class GlancesPlugin(object):
         try:
             return self._limits[plugin_name + '_' + value]
         except KeyError:
-            return []
+            return default
 
     def is_hide(self, value, header=""):
         """Return True if the value is in the hide configuration list.
