@@ -24,12 +24,13 @@ import warnings
 
 from glances.logger import logger
 from glances.compat import iteritems
-from glances.plugins.glances_batpercent import Plugin as BatPercentPlugin
-from glances.plugins.glances_hddtemp import Plugin as HddTempPlugin
+from glances.timer import Counter
+from glances.plugins.sensors.glances_batpercent import Plugin as BatPercentPlugin
+from glances.plugins.sensors.glances_hddtemp import Plugin as HddTempPlugin
 from glances.plugins.glances_plugin import GlancesPlugin
 
 SENSOR_TEMP_UNIT = 'C'
-SENSOR_FAN_UNIT = 'rpm'
+SENSOR_FAN_UNIT = 'R'
 
 
 def to_fahrenheit(celsius):
@@ -45,21 +46,30 @@ class Plugin(GlancesPlugin):
     The hard disks are already sorted by name.
     """
 
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the plugin."""
         super(Plugin, self).__init__(args=args,
+                                     config=config,
                                      stats_init_value=[])
 
+        start_duration = Counter()
+
         # Init the sensor class
+        start_duration.reset()
         self.glancesgrabsensors = GlancesGrabSensors()
+        logger.debug("Generic sensor plugin init duration: {} seconds".format(start_duration.get()))
 
         # Instance for the HDDTemp Plugin in order to display the hard disks
         # temperatures
-        self.hddtemp_plugin = HddTempPlugin(args=args)
+        start_duration.reset()
+        self.hddtemp_plugin = HddTempPlugin(args=args, config=config)
+        logger.debug("HDDTemp sensor plugin init duration: {} seconds".format(start_duration.get()))
 
         # Instance for the BatPercent in order to display the batteries
         # capacities
-        self.batpercent_plugin = BatPercentPlugin(args=args)
+        start_duration.reset()
+        self.batpercent_plugin = BatPercentPlugin(args=args, config=config)
+        logger.debug("Battery sensor plugin init duration: {} seconds".format(start_duration.get()))
 
         # We want to display the stat in the curse interface
         self.display_curse = True
@@ -122,6 +132,12 @@ class Plugin(GlancesPlugin):
 
             pass
 
+        # Set the alias for each stat
+        for stat in stats:
+            alias = self.has_alias(stat["label"].lower())
+            if alias:
+                stat["label"] = alias
+
         # Update the stats
         self.stats = stats
 
@@ -182,11 +198,7 @@ class Plugin(GlancesPlugin):
                 continue
             # New line
             ret.append(self.curse_new_line())
-            # Alias for the lable name ?
-            label = self.has_alias(i['label'].lower())
-            if label is None:
-                label = i['label']
-            msg = '{:{width}}'.format(label[:name_max_width],
+            msg = '{:{width}}'.format(i["label"][:name_max_width],
                                       width=name_max_width)
             ret.append(self.curse_add_line(msg))
             if i['value'] in (b'ERR', b'SLP', b'UNK', b'NOS'):
@@ -247,8 +259,9 @@ class GlancesGrabSensors(object):
 
         # !!! Disable Fan: High CPU consumption with psutil 5.2.0 or higher
         # Delete the two followings lines when corrected (https://github.com/giampaolo/psutil/issues/1199)
-        self.init_fan = False
-        logger.debug("Fan speed sensors disable (see https://github.com/giampaolo/psutil/issues/1199)")
+        # Correct and tested with PsUtil 5.6.1 (Ubuntu 18.04)
+        # self.init_fan = False
+        # logger.debug("Fan speed sensors disable (see https://github.com/giampaolo/psutil/issues/1199)")
 
         # Init the stats
         self.reset()
