@@ -268,7 +268,11 @@ class TestGlances(unittest.TestCase):
         if not is_admin():
             print("INFO: Not admin, SMART list should be empty")
             assert len(stats_grab) == 0
+        elif stats_grab == {}:
+            print("INFO: Admin but SMART list is empty")
+            assert len(stats_grab) == 0
         else:
+            print(stats_grab)
             self.assertTrue(stat in stats_grab[0].keys(), msg='Cannot find key: %s' % stat)
 
         print('INFO: SMART stats: %s' % stats_grab)
